/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998-2000  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * tap_dev.c,v 1.4 2006/12/11 10:39:30 mtbishop Exp
 */ 

/*
    This is an empty file.  All functionality formerly here is moved
    to the tun_dev.c file.  This file exists to prevent the automangle
    tools from moving a tap_dev.c from the generic directory - which
    we can't use - in place of this missing file.
 */
